// CD 11/2005
// survbatt_it.c
// N = (AN0/VDD)*1024
// Calculs pour VDD = 5v
// allumage PBO si AN0 = 4,9v soit N0 = 1003
//			PB1 si AN0 = 4,8v soit N1 = 983
//          PB2 si AN0 = 4,7v soit N2 = 962
//          PB3 si AN0 = 4,6v soit N3 = 942
// Le timer est en mode 16 bits avec horloge Fosc/4 soit 1MHz, prdiviseur par 16
// la priode des dbordement est donc 1uS * 16 * 65536 = 1.048576 S


#include <p18f452.h>
#include <stdio.h>  // pour fprintf
#include <xlcd.h>	// pour LCD sur PICDEM2+

#include "ftoa.c"


#define N0 1003
#define N1 983
#define N2 962
#define N3 942
#define S2 PORTAbits.RA4
#define q 4.8828e-3	// quantum pour un CAN 10bits 0v-5v

unsigned int vddmin=1023;
unsigned char chaine[10];	// pour ftoa

unsigned int mesVDD(void)
{
	ADCON0bits.GO_DONE=1;		// SOC sur AN0
	while(ADCON0bits.GO_DONE);	// attend EOC
	return (ADRES);
}

void surveille(void)
{
unsigned int mes;	// retour de conversion sur AN0
float vdd;			// valeur calculee de VDD

	mes=mesVDD();
	if (mes<N0) PORTBbits.RB0=1;
	if (mes<N1) PORTBbits.RB1=1;
	if (mes<N2) PORTBbits.RB2=1;
	if (mes<N3) PORTBbits.RB3=1;
	if (mes<vddmin)
		{
			vddmin=mes;	
			vdd=(float)vddmin*q;
			ftoa(vdd,chaine,3,'s');
			SetDDRamAddr(0);			//ligne 0 de l'afficheur
			fprintf (_H_USER, "VDD min: %s v",chaine);  
		}
}

void traiteIT(void); 		// prototype car le vecteur est dclar avant la sp d'IT


#pragma code it=0x08		// le vecteur d'IT prioritaire envoie  l'adresse 8
void saut_sur_spIT(void)	
{
_asm
	goto traiteIT			// saut crit en assembleur sur la fonction de traitement d'IT
_endasm
}
#pragma code				// le compilateur peut  nouveau grer les adresses


#pragma interrupt traiteIT	// SP d'IT 
void traiteIT(void)		
{
	if(INTCONbits.TMR0IF)			//vrifie un dbordement sur TMR0 
			{						
			INTCONbits.TMR0IF = 0;	//efface le drapeau d'IT
			surveille();			// surveille VDD 
			if (!S2) 	{
						PORTB=0;		// eteind les LEDs
						vddmin=1023;	// reinit VDD min
						SetDDRamAddr(0);	
						fprintf (_H_USER, "RAZ sur S2       ",chaine);
						}
			}
}

void main(void)
{
		ADCON0=1;		// CAN on. CLOCK=FOSC/2. CANAL0 (RA)
    	ADCON1=0x8E;
		OpenXLCD(FOUR_BIT & LINES_5X7 );	//initialise PIC pour LCD sur PD2
		SetDDRamAddr(0);	
		fprintf (_H_USER, "Surveille VDD    ",chaine);
		TRISB=0xF0;		// PRB0-PRB3 en sortie
		PORTB=0;		// LED eteintent
//		TRISA=0xFF;		// PORTA en entree  !!!! LE PORTA CONTROLE L'AFFICHEUR LCD SUR PD2+
		T0CON = 0b10000011;			//Active le TIMER0 - prescaler 1:16
		INTCONbits.TMR0IE = 1;		//Autorise interruption sur TMR0 
		INTCONbits.GIE = 1;			//autorise toutes les IT dmasques
		while(1);		
}
