#include <p18f452.h>
#include "initxlcd.c"
#include "ftoa.c"
#include "log.c"

#define Q 4.88e-3	// Quantum pour un CAN 10bits 0v 5v
#define BT25 11.58	// Rapport B/T25 de la thermistance (temprature en K)
#define B 3450.0	// Coeficient B
#define R25 470.0	// Valeur nominale  25 degrs centigrades
#define zero_abs	-273.0	// 0 absolue (0 Kelvin)
#define tmin	29.0
#define	tmax	31.0

rom const char efface[]="                ";	// efface une ligne LCD
rom const char degre[]=" deg";				// message
char chaine[15];							// tampon pour ftoa

// effectue une mesure sur CAN interne
unsigned int mesvolt(void)				
{
		ADCON0bits.GO_DONE=1;		// SOC
		while(ADCON0bits.GO_DONE);	// attend EOC
		return (ADRES);
}


#pragma interrupt traite_vect8
void traite_vect8(void)		
{
static char tictac=0;			// IT toutes les 500mS, il faut attendre 2 IT avant calcul
unsigned int m;
float Rn,Vt,t;

	if( PIR1bits.CCP1IF)				// l'IT provient d'une comparaison
			{	
			PIR1bits.CCP1IF=0;	//efface le drapeau d'IT					
			if (++tictac>=2) { 	
							   	tictac=0;
								m=mesvolt();
								Vt=(float)m*Q;			// tension en AN0
								Rn=(Vt*470.0)/(5.0-Vt);	// Rsistance de la CTN
								t=B/(log(Rn/R25)+BT25);	// temprature en K
								t+=zero_abs;			// temprature en C
								SetDDRamAddr(0);
								putrsXLCD(efface);		// efface LCD
								ftoa(t,chaine,2,'F');	// convertit en chaine
								SetDDRamAddr(0);	
								putsXLCD(chaine);		// affice temprature en degres C
								putrsXLCD(degre);		// affice degres
								if (t<tmin) PORTBbits.RB0=1; 	// chauffage
								if (t>tmax) PORTBbits.RB0=0; 	// arrt chauffage								
							}
			}
}			

#pragma code vecteur8=0x08
void sautversSPIT(void)
{
_asm goto traite_vect8 _endasm
}
#pragma code

void main(void)
{	
	DDRBbits.RB0=0;		// PB0 en sortie
	PORTBbits.RB0=0;	
	T1CON=0b10111001;	// configure le TIMER1
// configure le mode comparaison sur le TIMER1 avec IT sur CCP1 toutes les 62500 priodes de 8us soit 125ms
	T3CONbits.T3CCP2=0;	// mode comparaison entre TMR1 et CCPR1
	CCP1CON=0x0B;	// Trigger special event sur comparaison (RAZ TIMER1 lors de l'galit)
	CCPR1H=0x3d;	// galit aprs 62500 priodes 
	CCPR1L=0x09;
	PIE1bits.CCP1IE=1;	// active IT sur mode comparaison CCP1
	RCONbits.IPEN=1;		// Interruption prioritaires actives
	INTCONbits.GIE=1;		// Toutes les IT dmasques autorises

    OpenXLCD(FOUR_BIT & LINES_5X7 );

	ADCON0=0b00000001;		// CAN on. CLOCK=FOSC/2. 
	ADCON1=0x8E;			// justification  droite,  AN0 est activ, VREF+=VDD VREF-=VSS

	while(1);				// on attend
}
