#include <p18f452.h>
#include "initxlcd.c"
#include "ftoa.c"

#define q 4.8828e-3	// quantum pour un CAN 10bits 0v-5v
#define maxcoef 7

rom char efface[]="                ";
rom float coef[]={0.000896266134,0.072648700159,0.179833846396,0.268961064542,0.268961064542,0.179833846396,0.072648700159,0.000896266134};

float mes[8]={0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
char chaine[30];
float mesure;
char newmes=0;


// effectue une mesure sur CAN0 renvoie la valeur en volts
float mesvolt(void)
{
		ADCON0bits.GO_DONE=1;		// SOC
		while(ADCON0bits.GO_DONE);	// attend EOC
		return (((float)(ADRES))*q);
}

// sous programme d'interruption
#pragma interrupt vect18
void vect18(void)		
{
	static char i=0;
 	char j,k;
 	
	if( PIR1bits.CCP1IF)				// l'IT provient d'une comparaison ?
			{						
				mes[i]=mesvolt();
				k=i;
				mesure=0.0;
				for (j=0;j<=maxcoef;j++)
				{
					mesure=mesure+coef[j]*mes[k];
					k++;
					if (k > maxcoef) k=0;
				}
				i++;
				if (i > 7) i=0;
			}
	 PIR1bits.CCP1IF=0;	//efface le drapeau d'IT
	 newmes=1;
}			


#pragma code vecteur18=0x18
void vers_it18(void)
{
_asm goto vect18 _endasm
}
#pragma code


			
void main(void)
{	
// configure le TIMER1
	T1CONbits.RD16=0;	// TMR1 mode simple (pas de RW)
	T1CONbits.TMR1CS=0;	// compte les impulsions sur internal clock
	T1CONbits.T1CKPS1=1;	// prdiviseur =1/8 periode sortie = 8uS
	T1CONbits.T1CKPS0=1;	
	T1CONbits.T1SYNC=1;	// pas de synchronisation sur sleep/Reset	
	T1CONbits.TMR1ON=1;	// TMR1 Activ

// configure le mode comparaison sur le TIMER1 avec IT sur CCP1 toutes les 6250 priodes de 8us soit 50ms
// frquence d'chantillonnage : 20Hz
	T3CONbits.T3CCP2=0;	// mode comparaison entre TMR1 et CCPR1
	CCP1CON=0x0B;	// Trigger special event sur comparaison (RAZ TIMER1 lors de l'galit)

	CCPR1H=0x18;	// galit aprs 62500 priodes 
	CCPR1L=0x6A;

	PIE1bits.CCP1IE=1;	// active IT sur mode comparaison CCP1

	RCONbits.IPEN=1;		// Interruption prioritaires actives
	INTCONbits.GIE=1;		// Toutes les IT dmasques autorises

    OpenXLCD(FOUR_BIT & LINES_5X7 );
	
	ADCON0=1;				// CAN on. CLOCK=FOSC/2. CANAL0 (RA)
	ADCON1=0x8E;			// justification  droite, seul AN0 est activ, VREF+=VDD VREF-=VSS
	
	while(1)
	{	if (newmes)
		{	SetDDRamAddr(0);	// positionne le curseur en x,y	
			putrsXLCD(efface);	// efface LCD
			ftoa(mesure,chaine,0,'f');		// convertit en chaine
			SetDDRamAddr(0);	// positionne le curseur en x,y	
			putsXLCD(chaine);
			newmes=0;
		}
	}

}
