#include <p18cxxx.h>
#include <xlcd.h>

void Delay10TCYx(PARAM_SCLASS unsigned char);
void Delay100TCYx(PARAM_SCLASS unsigned char);
void Delay10KTCYx(PARAM_SCLASS unsigned char);
void Delay1KTCYx(PARAM_SCLASS unsigned char);

/********************************************************************
*       Function Name:  OpenXLCD  (Pour PICDEM2plus)                                  *
*       Return Value:   void                                        *
*       Parameters:     lcdtype: sets the type of LCD (lines)       *
*       Description:    This routine configures the LCD. Based on   *
*                       the Hitachi HD44780 LCD controller. The     *
*                       routine will configure the I/O pins of the  *
*                       microcontroller, setup the LCD for 4- or    *
*                       8-bit mode and clear the display. The user  *
*                       must provide three delay routines:          *
*                       DelayFor18TCY() provides a 18 Tcy delay     *
*                       DelayPORXLCD() provides at least 15ms delay *
*                       DelayXLCD() provides at least 5ms delay     *
********************************************************************/

void OpenXLCD(unsigned char lcdtype)
{
        // The data bits must be either a 8-bit port or the upper or
        // lower 4-bits of a port. These pins are made into inputs
    ADCON1bits.PCFG0 = 0;		/* Permettre au port RA de fonctionner en numrique */
    ADCON1bits.PCFG1 = 1;		/* sauf pour le Bit RA0 pour la conversion anal/num */
    ADCON1bits.PCFG2 = 1;
    ADCON1bits.PCFG3 = 1;
    TRISAbits.TRISA1 = 0;		/* Config des Bits du port RA1  RA3 en Sortie pour */
    TRISAbits.TRISA2 = 0;		/* l'afficheur LCD									*/
    TRISAbits.TRISA3 = 0;
    DDREbits.PSPMODE = 0;			/* Configure le port D en mode Entres / Sorties gnrales */ 

    TRIS_DATA_PORT &= 0xf0;
    DATA_PORT &= 0b00000011;
    
        TRIS_RW = 0;                    // All control signals made outputs
        TRIS_RS = 0;
        TRIS_E = 0;
        RW_PIN = 0;                     // R/W pin made low
        RS_PIN = 0;                     // Register select pin made low
        E_PIN = 0;                      // Clock pin made low

        // Delay for 15ms to allow for LCD Power on reset
   		DelayPORXLCD();
        
        // Setup interface to LCD
        TRIS_DATA_PORT &= 0xf0;
        DATA_PORT &= 0xf0;
        DATA_PORT |= 0x03;        // Function set cmd(4-bit interface)

        E_PIN = 1;                      // Clock the cmd in
        Delay100TCYx(1);
        E_PIN = 0;
        
        // Delay for at least 4.1ms
        DelayXLCD();

        // Setup interface to LCD
        DATA_PORT &= 0xf0;              // Function set cmd(4-bit interface)
        DATA_PORT |= 0x03;

        E_PIN = 1;                      // Clock the cmd in
        Delay10TCYx(5);
        E_PIN = 0;

        // Delay for at least 100us
        Delay100TCYx(20);

        // Setup interface to LCD
        DATA_PORT &= 0xf0;              // Function set cmd(4-bit interface)
        DATA_PORT |= 0x03;

        E_PIN = 1;                      // Clock cmd in
        Delay10TCYx(5);
        E_PIN = 0;

        TRIS_DATA_PORT |= 0xf0;         // Make data nibble output
        DATA_PORT &= 0xf0;              // Function set cmd(4-bit interface)
        DATA_PORT |= 0x02;

        E_PIN = 1;                      // Clock cmd in
        Delay10TCYx(5);
        E_PIN = 0;

        // Set data interface width, # lines, font
        while(BusyXLCD());              // Wait if LCD busy
        WriteCmdXLCD(0b00101000);          // Function set cmd

        // Turn the display on then off
        while(BusyXLCD());              // Wait if LCD busy
        WriteCmdXLCD(0b00001000);        // Display OFF/Blink OFF
        while(BusyXLCD());              // Wait if LCD busy
        WriteCmdXLCD(0b00001101);           // Display ON/Blink ON

        // Clear display
        while(BusyXLCD());              // Wait if LCD busy
        WriteCmdXLCD(0b00000110);             // Clear display

        // Set entry mode inc, no shift
        while(BusyXLCD());              // Wait if LCD busy
        WriteCmdXLCD(0b00000001);   // Entry Mode

        // Set DD Ram address to 0
        while(BusyXLCD());              // Wait if LCD busy
        SetDDRamAddr(0);                // Set Display data ram address to 0

        return;
}

